/*
 * File: split.c
 * -------------
 *
 * This program implements the function split that takes a sorted
 * unsigned integer array A and splits it into other unsigned integer
 * arrays consisting of subsets of A of consecutive integers. See the
 * comments just before the split function for details about how the
 * split is defined.
 *
 * Author: <GRAPSTE EDW TO ONOMATEPWNYMO SAS>
 * Student ID: <GRAPSTE EDW TON ARI8MO MHTRWOY SAS>
 * Class: EM102 - Spring Semester 2009
 * Homework #: 1
 */

#include <stdio.h>
#include <stdlib.h>

/*
 * Function: pretty_print_integer_set
 * Usage: pretty_print_integer_set(iset);
 * --------------------------------------
 * Prints the elements of an unsigned integer set in a pretty
 * way. We assume here the set is represented by an array, such that
 * the 0-th element of iset, i.e., iset[0], contains the number of
 * elements of the set, whereas elements iset[1] to iset[iset[0]] are
 * the true elements of the set.
 */
void pretty_print_integer_set(unsigned int *iset)
{
  if ( iset[0] == 0 ) { printf("{}"); }

  int i;

  printf("{");
  for (i = 1; i < iset[0]; ++i) {
    printf("%u, ", iset[i]);
  }
  printf("%u}", iset[iset[0]]);
}



/*
 * Function: split
 * Usage: sets = split(s, n, &nsets);
 * -----------------------------------
 *
 * <SYMPLHRWSTE TA SXOLIA>
 */
unsigned int** split(unsigned int *s, unsigned int *nsets)
{
  /*
   * AYTH EINAI H SYNARTHSH POU PREPEI NA SYMPLHRWSETE
   * TIS PARAKATW DYO ENTOLES MPOREITE NA TIS ALLAKSETE
   */
  *nsets = 0;
  return NULL;
}


/*
 * Function: read_iset
 * Usage: success = read_iset(infile, iset, max_size);
 * ---------------------------------------------------
 * Read the elements of an unsigned integer set from the file stream
 * "infile" and stores them in the given unsigned integer set "iset".
 * The variable "max_size" indicates the maximum possible number of
 * elements in the integer set "iset".
 * The function returns 1 upon successful assignment to the set, 0
 * otherwise.
 */
int read_integer_set(FILE* infile, unsigned int *iset, unsigned int max_size)
{
  unsigned int size, i;

  /* first read the number of elements in the set */
  fscanf(infile, "%u", &size);

  /* if number of elements is too big return 0 */
  if ( size > max_size ) { return 0; }
  iset[0] = size;

  /* read the elements of the set one-by-one */
  for (i = 1; i <= size; ++i) {
    fscanf(infile, "%u", &iset[i]);
  }

  return 1;
}


int main(int argc, char **argv)
{
  unsigned int ia[1001];

  /* print program usage if number of command line arguments is too
     small */
  if ( argc < 2 ) {
    printf("\tUsage: %s <input file>\n", argv[0]);
    return 0;
  }

  /* open file for reading input */
  FILE* infile = fopen(argv[1], "r");
  if ( infile == NULL ) {
    printf("Could not open file: %s\n", argv[1]);
    return 1;
  }

  /* read number of integer sets in file */
  unsigned int n, i;
  fscanf(infile, "%u", &n);

  /* read the integer sets one-by-one and slit them */
  for (i = 1; i <= n; ++i) {
    int result = read_integer_set(infile, ia, 1000);
    if ( !result ) {
      printf("Integer set read is too big!\n");
      return 2;
    }
    unsigned int nsets;
    unsigned int **isets = split(ia, &nsets);

    if ( isets != NULL ) {
      unsigned int k;
      printf("Set #%u: ", i);
      pretty_print_integer_set(ia);
      printf(":\n{");
      for (k = 0; k < nsets - 1; ++k) {
	pretty_print_integer_set(isets[k]);
	printf(", ");
      }
      pretty_print_integer_set(isets[nsets - 1]);  
      printf("}\n\n");

      /* free allocated memory */
      for (k = 0; k < nsets; ++k) {
	free(isets[k]);
      }
      free(isets);
    }
  }

  /* close the input file */
  fclose(infile);

  return 0;
}
